﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Linq.Expressions;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WpfPlayground
{
   /// <summary>
   /// Interaction logic for NumericUpDown.xaml
   /// </summary>
   public partial class NumericUpDown : UserControl, INotifyPropertyChanged
   {
      public static readonly DependencyProperty ValueProperty =
         DependencyProperty.Register("Value", typeof(double), typeof(NumericUpDown));

      public ICommand UpCommand { get; }
      public ICommand DownCommand { get; }

      private double _value;
      public double Value
      {
         get
         {
            return (double)GetValue(ValueProperty);
         }
         set
         {
            SetValue(ValueProperty, value);
         } 
      }

      public NumericUpDown()
      {
         InitializeComponent();
         UpCommand = new RelayCommand(o => Up());
         DownCommand = new RelayCommand(o => Down());
      }

      private void Up()
      {
         Value++;
      }

      private void Down()
      {
         Value--;
      }

      #region PC

      public event PropertyChangedEventHandler PropertyChanged;

      protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
      {
         PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
      }

      protected virtual void OnPropertyChanged<T>(Expression<Func<T>> raiser)
      {
         var propName = ((MemberExpression)raiser?.Body)?.Member.Name;
         OnPropertyChanged(propName);
      }

      protected bool SetProperty<T>(ref T field, T value, [CallerMemberName] string name = null)
      {
         if (EqualityComparer<T>.Default.Equals(field, value)) return false;
         field = value;
         OnPropertyChanged(name);
         return true;
      }

      #endregion

      private void ButtonBase_OnClick(object sender, RoutedEventArgs e)
      {
         Value++;
      }
      private void ButtonBase2_OnClick(object sender, RoutedEventArgs e)
      {
         Value--;
      }
   }
}
